@@ -4,7 +4,7 @@ use core::marker;
 use crate::integer_encoding::{IntegerEncoding, UsizeEncoding};
 use musli::de::{Decoder, PackDecoder, PairDecoder, PairsDecoder, SequenceDecoder, ValueVisitor};
 use musli::error::Error;
-use musli_binary_common::reader::PositionedReader;
+use musli_common::reader::PositionedReader;
 
 /// A very simple decoder suitable for storage decoding.
 pub struct StorageDecoder<R, I, L>
@@ -52,14 +52,14 @@ where
     L: UsizeEncoding,
 {
     type Error = R::Error;
-    type Pack<'this> = Self;
+    type Pack = Self;
     type Some = Self;
-    type Sequence<'this> = LimitedStorageDecoder<R, I, L>;
-    type Tuple<'this> = Self;
-    type Map<'this> = LimitedStorageDecoder<R, I, L>;
-    type Struct<'this> = LimitedStorageDecoder<R, I, L>;
-    type TupleStruct<'this> = LimitedStorageDecoder<R, I, L>;
-    type Variant<'this> = Self;
+    type Sequence = LimitedStorageDecoder<R, I, L>;
+    type Tuple = Self;
+    type Map = LimitedStorageDecoder<R, I, L>;
+    type Struct = LimitedStorageDecoder<R, I, L>;
+    type TupleStruct = LimitedStorageDecoder<R, I, L>;
+    type Variant = Self;
 
     #[inline]
     fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
@@ -82,11 +82,8 @@ where
     }
 
     #[inline]
-    fn decode_pack<T, O>(self, decoder: T) -> Result<O, Self::Error>
-    where
-        T: FnOnce(Self::Pack<'_>) -> Result<O, Self::Error>,
-    {
-        decoder(self)
+    fn decode_pack(self) -> Result<Self::Pack, Self::Error> {
+        Ok(self)
     }
 
     #[inline]
@@ -251,43 +248,28 @@ where
     }
 
     #[inline]
-    fn decode_sequence<T, O>(self, decoder: T) -> Result<O, Self::Error>
-    where
-        T: FnOnce(Self::Sequence<'_>) -> Result<O, Self::Error>,
-    {
-        decoder(LimitedStorageDecoder::new(self)?)
+    fn decode_sequence(self) -> Result<Self::Sequence, Self::Error> {
+        LimitedStorageDecoder::new(self)
     }
 
     #[inline]
-    fn decode_tuple<T, O>(self, _: usize, decoder: T) -> Result<O, Self::Error>
-    where
-        T: FnOnce(Self::Tuple<'_>) -> Result<O, Self::Error>,
-    {
-        decoder(self)
+    fn decode_tuple(self, _: usize) -> Result<Self::Tuple, Self::Error> {
+        Ok(self)
     }
 
     #[inline]
-    fn decode_map<T, O>(self, decoder: T) -> Result<O, Self::Error>
-    where
-        T: FnOnce(Self::Map<'_>) -> Result<O, Self::Error>,
-    {
-        decoder(LimitedStorageDecoder::new(self)?)
+    fn decode_map(self) -> Result<Self::Map, Self::Error> {
+        LimitedStorageDecoder::new(self)
     }
 
     #[inline]
-    fn decode_struct<T, O>(self, _: usize, decoder: T) -> Result<O, Self::Error>
-    where
-        T: FnOnce(Self::Struct<'_>) -> Result<O, Self::Error>,
-    {
-        decoder(LimitedStorageDecoder::new(self)?)
+    fn decode_struct(self, _: usize) -> Result<Self::Struct, Self::Error> {
+        LimitedStorageDecoder::new(self)
     }
 
     #[inline]
-    fn decode_tuple_struct<T, O>(self, _: usize, decoder: T) -> Result<O, Self::Error>
-    where
-        T: FnOnce(Self::TupleStruct<'_>) -> Result<O, Self::Error>,
-    {
-        decoder(LimitedStorageDecoder::new(self)?)
+    fn decode_tuple_struct(self, _: usize) -> Result<Self::TupleStruct, Self::Error> {
+        LimitedStorageDecoder::new(self)
     }
 
     #[inline]
@@ -296,11 +278,8 @@ where
     }
 
     #[inline]
-    fn decode_variant<T, O>(self, decoder: T) -> Result<O, Self::Error>
-    where
-        T: FnOnce(Self::Variant<'_>) -> Result<O, Self::Error>,
-    {
-        decoder(self)
+    fn decode_variant(self) -> Result<Self::Variant, Self::Error> {
+        Ok(self)
     }
 }
 
@@ -393,7 +372,7 @@ where
 {
     type Error = R::Error;
     type First<'this> = StorageDecoder<&'this mut R, I, L> where Self: 'this;
-    type Second<'this> = StorageDecoder<&'this mut R, I, L> where Self: 'this;
+    type Second = StorageDecoder<R, I, L>;
 
     #[inline]
     fn first(&mut self) -> Result<Self::First<'_>, Self::Error> {
@@ -401,12 +380,12 @@ where
     }
 
     #[inline]
-    fn second(&mut self) -> Result<Self::Second<'_>, Self::Error> {
-        Ok(StorageDecoder::new(&mut self.reader))
+    fn second(self) -> Result<Self::Second, Self::Error> {
+        Ok(self)
     }
 
     #[inline]
-    fn skip_second(&mut self) -> Result<bool, Self::Error> {
+    fn skip_second(self) -> Result<bool, Self::Error> {
         Ok(false)
     }
 }
